// Copyright 2014 Google Inc.

#import <Foundation/Foundation.h>

#import "GoogleCastGames/GCKGameManagerState.h"

#import "GCKGameplayState.h"
#import "GCKLobbyState.h"
#import "GCKPlayerState.h"

/**
 * @file GCKGameManagerChannel.h
 * Contains GCKGameManagerChannel and GCKGameManagerChannelDelegate definitions
 */

@class GCKGameManagerResult;

@protocol GCKGameManagerChannelDelegate;

/**
 * A CastChannel for game control operations.
 *
 * @ingroup Games
 */
GCK_EXPORT
@interface GCKGameManagerChannel : GCKCastChannel

/**
 * The delegate for receiving notifications from the GCKGameManagerChannel.
 */
@property(nonatomic, weak) id<GCKGameManagerChannelDelegate> delegate;

/**
 * The current state of the game manager.
 * @synchronized
 */
@property(nonatomic, readonly) GCKGameManagerState *currentState;

/**
 * The last controllable player ID used in this session. This can be <code>nil</code>
 * if there is no controllable player set up with the receiver.
 *
 * @see sendPlayerAvailableRequest
 */
@property(nonatomic, readonly) NSString *lastUsedPlayerID;

/**
 * Whether this channel is connected with the receiver's game manager and we are ready to interact
 * with it.  This method will return NO from the point of creation of this channel.
 * @return YES if this channel is connected and the GameManager and ready to interact with. NO
 * otherwise.
 **/
@property(nonatomic, readonly) BOOL isInitialConnectionEstablished;

/**
 * Designated initializer. Automatically connects to receiver GameManager.
 *
 * @param castSessionID SessionID corresponding to currently connected GCKDeviceManager.
 */
- (instancetype)initWithSessionID:(NSString *)castSessionID NS_DESIGNATED_INITIALIZER;

/**
 * Sends a request to the receiver to put the last used player on this sender into the
 * {@link GCKPlayerStateAvailable} state. If this is not a valid transition for that player
 * an error will be triggered in the {@link GCKGameManagerChannelDelegate}.
 * If there is no last used player, a new player will be registered and its player ID will be set as
 * the last used player ID when the receiver responds to this request.
 * This method should only be called after <code>gameManagerChannelDidConnect:</code> delegate
 * callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the game manager running on the receiver.
 *
 * @see lastUsedPlayerID
 *
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendPlayerAvailableRequest:(id)extraData;

/**
 * Sends a request to the receiver to put the player into the {@link GCKPlayerStateAvailable}
 * state. If this is not a valid transition for that player an error will be triggered
 * in the {@link GCKGameManagerChannelDelegate}.
 * If the playerId is <code>nil</code> a new player will be registered and its player ID will
 * be set as the last used player ID when the receiver responds to this request.
 * This method should only be called after <code>gameManagerChannelDidConnect:</code> delegate
 * callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the game manager running on the receiver.
 *
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @param playerID The player ID of the player to change the state.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendPlayerAvailableRequest:(id)extraData
                               playerID:(NSString *)playerID;

/**
 * Sends a request to the receiver to put the last used player on this sender into the
 * {@link GCKPlayerStateReady} state. If this is not a valid transition for that player or if
 * there is no last used player ID, an error will be triggered in the
 * {@link GCKGameManagerChannelDelegate}.
 * This method should only be called after <code>gameManagerChannelDidConnect:</code> delegate
 * callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the game manager running on the receiver.
 *
 * @see lastUsedPlayerID
 *
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendPlayerReadyRequest:(id)extraData;

/**
 * Sends a request to the receiver to put the player into the {@link GCKPlayerStateReady}
 * state. If this is not a valid transition for that player or if there is no last used player ID,
 * an error will be triggered in the {@link GCKGameManagerChannelDelegate}.
 * This method should only be called after <code>gameManagerChannelDidConnect:</code> delegate
 * callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the game manager running on the receiver.
 *
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @param playerID The player ID of the player to change the state.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendPlayerReadyRequest:(id)extraData
                           playerID:(NSString *)playerID;

/**
 * Sends a request to the receiver to put the last used player on this sender into the
 * {@link GCKPlayerStatePlaying} state. If this is not a valid transition for that player or if
 * there is no last used player ID, an error will be triggered in the
 * {@link GCKGameManagerChannelDelegate}.
 * This method should only be called after <code>gameManagerChannelDidConnect:</code> delegate
 * callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the game manager running on the receiver.
 *
 * @see lastUsedPlayerID
 *
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendPlayerPlayingRequest:(id)extraData;

/**
 * Sends a request to the receiver to put the player into the {@link GCKPlayerStatePlaying}
 * state. If this is not a valid transition for that player or if there is no last used player ID,
 * an error will be triggered in the {@link GCKGameManagerChannelDelegate}.
 * This method should only be called after <code>gameManagerChannelDidConnect:</code> delegate
 * callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the game manager running on the receiver.
 *
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @param playerID The player ID of the player to change the state.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendPlayerPlayingRequest:(id)extraData
                             playerID:(NSString *)playerID;

/**
 * Sends a request to the receiver to put the last used player on this sender into the
 * {@link GCKPlayerStateIdle} state. If this is not a valid transition for that player or if
 * there is no last used player ID, an error will be triggered in the
 * {@link GCKGameManagerChannelDelegate}.
 * This method should only be called after <code>gameManagerChannelDidConnect:</code> delegate
 * callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the game manager running on the receiver.
 *
 * @see lastUsedPlayerID
 *
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendPlayerIdleRequest:(id)extraData;

/**
 * Sends a request to the receiver to put the player into the {@link GCKPlayerStateIdle}
 * state. If this is not a valid transition for that player or if there is no last used player ID,
 * an error will be triggered in the {@link GCKGameManagerChannelDelegate}.
 * This method should only be called after <code>gameManagerChannelDidConnect:</code> delegate
 * callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the game manager running on the receiver.
 *
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @param playerID The player ID of the player to change the state.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendPlayerIdleRequest:(id)extraData
                          playerID:(NSString *)playerID;

/**
 * Sends a request to the receiver to put the last used player on this sender into the
 * {@link GCKPlayerStateQuit} state. If this is not a valid transition for that player or if
 * there is no last used player ID, an error will be triggered in the
 * {@link GCKGameManagerChannelDelegate}.
 * This method should only be called after <code>gameManagerChannelDidConnect:</code> delegate
 * callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the game manager running on the receiver.
 *
 * @see lastUsedPlayerID
 *
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendPlayerQuitRequest:(id)extraData;

/**
 * Sends a request to the receiver to put the player into the {@link GCKPlayerStateQuit}
 * state. If this is not a valid transition for that player or if there is no last used player ID,
 * an error will be triggered in the {@link GCKGameManagerChannelDelegate}.
 * This method should only be called after <code>gameManagerChannelDidConnect:</code> delegate
 * callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the game manager running on the receiver.
 *
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @param playerID The player ID of the player to change the state.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendPlayerQuitRequest:(id)extraData
                          playerID:(NSString *)playerID;

/**
 * Sends a game specific message to the receiver, its contents are entirely up to the
 * application. The message will originate from lastUsedPlayerID.
 * The receiver will send a response back to this sender via the GCKGameManagerChannelDelegate.
 * This function should only be called after <code>gameManagerChannelDidConnect:</code>
 * delegate callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the receiver GameManager.
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendGameRequest:(id)extraData;

/**
 * Sends a game specific message to the receiver, its contents are entirely up to the
 * application. The message will originate from playerID. If playerID is nil
 * lastUsedPlayerID will be used. The receiver will send a response back to
 * this sender via the GCKGameManagerChannelDelegate.
 * This function should only be called after <code>gameManagerChannelDidConnect:</code>
 * delegate callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the receiver GameManager.
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @param playerID The id of the controllable player sending this message.
 * @return The request ID, or <code>kGCKInvalidRequestID</code> if the request could not be sent.
 */
- (NSInteger)sendGameRequest:(id)extraData playerID:(NSString *)playerID;

/**
 * Sends a game specific message to the receiver, its contents are entirely up to the
 * application. The message will originate from lastUsedPlayerID. This is a fire and
 * forget method where there's no guarantee the message is sent and the receiver won't send a
 * response back to this sender.
 * This function should only be called after <code>gameManagerChannelDidConnect:</code>
 * delegate callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the receiver GameManager.
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 */
- (void)sendGameMessage:(id)extraData;

/**
 * Sends a game specific message to the receiver, its contents are entirely up to the
 * application. The message will originate from the specified local playerID. This is a fire
 * and forget method where there's no guarantee the message is sent and the receiver
 * won't send a response back to this sender.
 * This function should only be called after <code>gameManagerChannelDidConnect:</code>
 * delegate callback has been executed.
 * Triggers <code>gameManagerChannel:requestDidFailWithID:error:</code> if GCKGameManagerChannel
 * is not yet connected to the receiver GameManager.
 * @param extraData Custom application-specific data to pass along with the request. Must either
 * be an object that can be serialized to JSON using NSJSONSerialization, or nil.
 * @param playerID The ID of the controllable player sending this message.
 */
- (void)sendGameMessage:(id)extraData playerID:(NSString *)playerID;

@end

/**
 * The delegate for GCKGameManagerChannel notifications.
 */
GCK_EXPORT
@protocol GCKGameManagerChannelDelegate <NSObject>

/**
 * Called when the {@link GCKGameManagerState} has changed.
 *
 * @param gameManagerChannel Affected GCKGameManagerChannel.
 * @param currentState The current {@link GCKGameManagerState}.
 * @param previousState The previous {@link GCKGameManagerState}.
 */
- (void)gameManagerChannel:(GCKGameManagerChannel *)gameManagerChannel
          stateDidChangeTo:(GCKGameManagerState *)currentState
                      from:(GCKGameManagerState *)previousState;

/**
 * Called when the receiver sends a game message for a specific player.
 *
 * @param gameManagerChannel Affected GCKGameManagerChannel.
 * @param gameMessage Game message sent from the receiver.
 * @param playerID The player ID associated with the gameMessage.
 */
- (void)gameManagerChannel:(GCKGameManagerChannel *)gameManagerChannel
     didReceiveGameMessage:(id)gameMessage
               forPlayerID:(NSString *)playerID;

/**
 * Called when a player request or game request was successful.
 *
 * @param gameManagerChannel Affected GCKGameManagerChannel.
 * @param requestID The request ID that failed. This is the ID returned when the request was made.
 * @param result {@link GCKMangerResult} returned as part of this request response.
 */
- (void)gameManagerChannel:(GCKGameManagerChannel *)gameManagerChannel
   requestDidSucceedWithID:(NSInteger)requestID
                    result:(GCKGameManagerResult *)result;

/**
 * Called when a player request or game request failed with an error.
 *
 * @param gameManagerChannel Affected GCKGameManagerChannel.
 * @param requestID The request ID that failed. This is the ID returned when the request was made.
 * @param error The <code>NSError</code> associated with the failure.
 */
- (void)gameManagerChannel:(GCKGameManagerChannel *)gameManagerChannel
      requestDidFailWithID:(NSInteger)requestID
                     error:(NSError *)error;

@required

/**
 * Called when the receiver's game manager connects successfully and we are ready to interact with
 * it.
 *
 * @param gameManagerChannel Affected GCKGameManagerChannel.
 */
- (void)gameManagerChannelDidConnect:(GCKGameManagerChannel *)gameManagerChannel;

/**
 * Called when the receiver's game manager encounters an error during connection.
 *
 * @param gameManagerChannel Affected GCKGameManagerChannel.
 * @param error The <code>NSError</code> associated with the failure.
 */
- (void)gameManagerChannel:(GCKGameManagerChannel *)gameManagerChannel
 didFailToConnectWithError:(NSError *)error;

@end
